/*	PG_PARSE.H
	Copyright (c) 1992	Keith L. Robertson	All Rights Reserved

	Declares types and functions needed to compile
	the parser and parsing tables.
	Also declare parse (), called by driver program, to parse the input.

	Any user of this file must type-define LEX_INFO
	before #including it.
*/
#ifndef PG_PARSE_H
#define	PG_PARSE_H

#ifndef NUMBERS_H
#include <numbers.h>
#endif


/* The point of all this... Call parse () to parse the input. */
void	parse ();

/* Used by the semantic actions to access symbol attributes. */
extern struct LEX_INFO*	attr;

/* Used by error semantic actions to advance the input
   stream to a particular token or set of tokens. */
void	skip_to_token (short tok);
void	skip_to_bittok (char* tokmap);


/* ------------------------------------- */

typedef void	(*SEM_ACTION) ();

struct	PRODUCTION_INFO	{	/* Indexed by production number. */
    uchar	nonterm;		/* Nonterminal leading the production */
    schar	size_m1;		/* rhs.count() - 1  */
    SEM_ACTION	sem_action;		/* Semantic action. */
};


/* ------------------------------------- */

typedef ushort	ACTION;
#define		action_part(act)	(act >> 14)
#define		second_part(act)	(act & 0x3fff)
#define		action_mix(act,sec)	((act << 14) | sec)

#define		ERROR_ACT	0
#define		ACCEPT_ACT	1
#define		SHIFT_ACT	2
#define		REDUCE_ACT	3

#define		error(sec)	action_mix(ERROR_ACT,sec)
#define		accept(sec)	action_mix(ACCEPT_ACT,sec)
#define		shift(sec)	action_mix(SHIFT_ACT,sec)
#define		reduce(sec)	action_mix(REDUCE_ACT,sec)
#define		no_change(sec)	action_mix(SHIFT_ACT,0)


struct	ACTION_ITEM	{
    ushort	token;
    ACTION	action;
};
#define		BM		0x4000		/* Lookup token by bitmap */
#define		ERR_TK		0x4ffe
#define		ANY_TK		0x4fff
#define		BR		0x8000		/* BRanch */
/* if (token & BR), branch to ACTION_ITEM (token^BR)
			and on ANY_TK do the given action. */

/* Must preserve the order:  BM < ERR_TK < ANY_TK < BR.
   BR must have a bit set that will be clear in all other tokens,
   and there must be room beneath for any item set, or state, number. 
*/


/* ------------------------------------- */

struct	GOTO_LIST_ITEM		{
    ushort	cur_state;
    ushort	nxt_state;
};
#define		ANY_ST		0x7fff
#define		BG		0x8000		/* Lookup cur_state by bitmap */

#endif PG_PARSE_H
